@@ -27,19 +27,6 @@ const appSys = Shell.AppSystem.get_default();
 const APPLICATION_ICON_SIZE = 32;
 const MENU_HEIGHT_OFFSET = 132;
 
-function fixMarkup(text, allowMarkup) {
-    if (allowMarkup) {
-        let _text = text.replace(/&(?!amp;|quot;|apos;|lt;|gt;)/g, '&amp;');
-        _text = _text.replace(/<(?!\/?[biu]>)/g, '&lt;');
-        try {
-            Pango.parse_markup(_text, -1, '');
-            return _text;
-        } catch (e) {
-        }
-    }
-    return GLib.markup_escape_text(text, -1);
-}
-
 const ActivitiesMenuItem = new Lang.Class({
     Name: 'ActivitiesMenuItem',
     Extends: PopupMenu.PopupBaseMenuItem,
@@ -69,8 +56,9 @@ const ApplicationMenuItem = new Lang.Class({
         let icon = this._app.create_icon_texture(APPLICATION_ICON_SIZE);
 	this.addActor(icon);
 
-        let appName = fixMarkup(this._app.get_name());
-        this.addActor(new St.Label({ text: appName }));
+        let appLabel = new St.Label({ text: app.get_name() });
+        this.addActor(appLabel, { span: -1, expand: true });
+        this.actor.label_actor = appLabel;
     },
 
     activate: function(event) {
@@ -84,6 +72,10 @@ const ApplicationMenuItem = new Lang.Class({
         if (active)
             this._button.scrollToButton(this);
         this.parent(active, params);
+    },
+
+    _getPreferredWidth: function(actor, forHeight, alloc) {
+        alloc.min_size = alloc.natural_size = -1;
     }
 });
 
@@ -124,8 +116,8 @@ const HotCorner = new Lang.Class({
     Name: 'HotCorner',
     Extends: Layout.HotCorner,
 
-    _init : function() {
-        this.parent();
+    _init : function(layoutManager) {
+        this.parent(layoutManager);
     },
 
     _onCornerEntered : function() {
@@ -178,7 +170,7 @@ const ApplicationsButton = new Lang.Class({
 
     _init: function() {
         this.parent(1.0, null, false);
-        this._hotCorner = new HotCorner();
+        this._hotCorner = new HotCorner(Main.layoutManager);
         this.setMenu(new ApplicationsMenu(this.actor, 1.0, St.Side.TOP, this, this._hotCorner));
         Main.panel.menuManager.addMenu(this.menu);
 
@@ -318,21 +310,18 @@ const ApplicationsButton = new Lang.Class({
                 if (!entry.get_app_info().get_nodisplay()) {
                     let app = appSys.lookup_app_by_tree_entry(entry);
                     let menu_id = dir.get_menu_id();
-                    if (!this.applicationsByCategory[menu_id])
-                        this.applicationsByCategory[menu_id] = new Array();
                     this.applicationsByCategory[menu_id].push(app);
                 }
             } else if (nextType == GMenu.TreeItemType.DIRECTORY) {
                 let subdir = iter.get_directory();
-                if (subdir.get_is_nodisplay())
-                    continue;
-
-                let menu_id = subdir.get_menu_id();
-                this.applicationsByCategory[menu_id] = new Array();
-                this._loadCategory(subdir);
-                if (this.applicationsByCategory[menu_id].length > 0) {
-                   let categoryMenuItem = new CategoryMenuItem(this, subdir);
-                   this.categoriesBox.add_actor(categoryMenuItem.actor);
+                if (!subdir.get_is_nodisplay()) {
+                    let menu_id = subdir.get_menu_id();
+                    this.applicationsByCategory[menu_id] = new Array();
+                    this._loadCategory(subdir);
+                    if (this.applicationsByCategory[menu_id].length > 0) {
+                        let categoryMenuItem = new CategoryMenuItem(this, subdir);
+                        this.categoriesBox.add_actor(categoryMenuItem.actor);
+                    }
                 }
             }
         }
@@ -431,15 +420,14 @@ const ApplicationsButton = new Lang.Class({
         while ((nextType = iter.next()) != GMenu.TreeItemType.INVALID) {
             if (nextType == GMenu.TreeItemType.DIRECTORY) {
                 let dir = iter.get_directory();
-                if (dir.get_is_nodisplay())
-                    continue;
-
-                let menu_id = dir.get_menu_id();
-                this.applicationsByCategory[menu_id] = new Array();
-                this._loadCategory(dir);
-                if (this.applicationsByCategory[menu_id].length > 0) {
-                   let categoryMenuItem = new CategoryMenuItem(this, dir);
-                   this.categoriesBox.add_actor(categoryMenuItem.actor);
+                if (!dir.get_is_nodisplay()) {
+                    let menu_id = dir.get_menu_id();
+                    this.applicationsByCategory[menu_id] = new Array();
+                    this._loadCategory(dir);
+                    if (this.applicationsByCategory[menu_id].length > 0) {
+                        let categoryMenuItem = new CategoryMenuItem(this, dir);
+                        this.categoriesBox.add_actor(categoryMenuItem.actor);
+                    }
                 }
             }
         }
